import sys
import typing
import bpy.types


def add_marker(location: typing.List[float] = (0.0, 0.0)):
    ''' Place new marker at specified location

    :param location: Location, Location of marker on frame
    :type location: typing.List[float]
    '''

    pass


def add_marker_at_click():
    ''' Place new marker at the desired (clicked) position

    '''

    pass


def add_marker_move(CLIP_OT_add_marker=None, TRANSFORM_OT_translate=None):
    ''' Add new marker and move it on movie

    :param CLIP_OT_add_marker: Add Marker, Place new marker at specified location
    :param TRANSFORM_OT_translate: Move, Move selected items
    '''

    pass


def add_marker_slide(CLIP_OT_add_marker=None, TRANSFORM_OT_translate=None):
    ''' Add new marker and slide it with mouse until mouse button release

    :param CLIP_OT_add_marker: Add Marker, Place new marker at specified location
    :param TRANSFORM_OT_translate: Move, Move selected items
    '''

    pass


def apply_solution_scale(distance: float = 0.0):
    ''' Apply scale on solution itself to make distance between selected tracks equals to desired

    :param distance: Distance, Distance between selected tracks
    :type distance: float
    '''

    pass


def average_tracks(keep_original: bool = True):
    ''' Average selected tracks into active

    :param keep_original: Keep Original, Keep original tracks
    :type keep_original: bool
    '''

    pass


def bundles_to_mesh():
    ''' Create vertex cloud using coordinates of reconstructed tracks :file: startup/bl_operators/clip.py\:282 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/clip.py$282> _

    '''

    pass


def camera_preset_add(name: str = "",
                      remove_name: bool = False,
                      remove_active: bool = False,
                      use_focal_length: bool = True):
    ''' Add or remove a Tracking Camera Intrinsics Preset

    :param name: Name, Name of the preset, used to make the path name
    :type name: str
    :param remove_name: remove_name
    :type remove_name: bool
    :param remove_active: remove_active
    :type remove_active: bool
    :param use_focal_length: Include Focal Length, Include focal length into the preset
    :type use_focal_length: bool
    '''

    pass


def change_frame(frame: int = 0):
    ''' Interactively change the current frame number

    :param frame: Frame
    :type frame: int
    '''

    pass


def clean_tracks(frames: int = 0,
                 error: float = 0.0,
                 action: typing.Union[str, int] = 'SELECT'):
    ''' Clean tracks with high error values or few frames

    :param frames: Tracked Frames, Effect on tracks which are tracked less than specified amount of frames
    :type frames: int
    :param error: Reprojection Error, Effect on tracks which have got larger reprojection error
    :type error: float
    :param action: Action, Cleanup action to execute * SELECT Select -- Select unclean tracks. * DELETE_TRACK Delete Track -- Delete unclean tracks. * DELETE_SEGMENTS Delete Segments -- Delete unclean segments of tracks.
    :type action: typing.Union[str, int]
    '''

    pass


def clear_solution():
    ''' Clear all calculated data

    '''

    pass


def clear_track_path(action: typing.Union[str, int] = 'REMAINED',
                     clear_active: bool = False):
    ''' Clear tracks after/before current position or clear the whole track

    :param action: Action, Clear action to execute * UPTO Clear Up To -- Clear path up to current frame. * REMAINED Clear Remained -- Clear path at remaining frames (after current). * ALL Clear All -- Clear the whole path.
    :type action: typing.Union[str, int]
    :param clear_active: Clear Active, Clear active track only instead of all selected tracks
    :type clear_active: bool
    '''

    pass


def constraint_to_fcurve():
    ''' Create F-Curves for object which will copy object's movement caused by this constraint :file: startup/bl_operators/clip.py\:520 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/clip.py$520> _

    '''

    pass


def copy_tracks():
    ''' Copy selected tracks to clipboard

    '''

    pass


def create_plane_track():
    ''' Create new plane track out of selected point tracks

    '''

    pass


def cursor_set(location: typing.List[float] = (0.0, 0.0)):
    ''' Set 2D cursor location

    :param location: Location, Cursor location in normalized clip coordinates
    :type location: typing.List[float]
    '''

    pass


def delete_marker():
    ''' Delete marker for current frame from selected tracks

    '''

    pass


def delete_proxy():
    ''' Delete movie clip proxy files from the hard drive :file: startup/bl_operators/clip.py\:349 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/clip.py$349> _

    '''

    pass


def delete_track():
    ''' Delete selected tracks

    '''

    pass


def detect_features(placement: typing.Union[str, int] = 'FRAME',
                    margin: int = 16,
                    threshold: float = 0.5,
                    min_distance: int = 120):
    ''' Automatically detect features and place markers to track

    :param placement: Placement, Placement for detected features * FRAME Whole Frame -- Place markers across the whole frame. * INSIDE_GPENCIL Inside Annotated Area -- Place markers only inside areas outlined with the Annotation tool. * OUTSIDE_GPENCIL Outside Annotated Area -- Place markers only outside areas outlined with the Annotation tool.
    :type placement: typing.Union[str, int]
    :param margin: Margin, Only features further than margin pixels from the image edges are considered
    :type margin: int
    :param threshold: Threshold, Threshold level to consider feature good enough for tracking
    :type threshold: float
    :param min_distance: Distance, Minimal distance accepted between two features
    :type min_distance: int
    '''

    pass


def disable_markers(action: typing.Union[str, int] = 'DISABLE'):
    ''' Disable/enable selected markers

    :param action: Action, Disable action to execute * DISABLE Disable -- Disable selected markers. * ENABLE Enable -- Enable selected markers. * TOGGLE Toggle -- Toggle disabled flag for selected markers.
    :type action: typing.Union[str, int]
    '''

    pass


def dopesheet_select_channel(location: typing.List[float] = (0.0, 0.0),
                             extend: bool = False):
    ''' Select movie tracking channel

    :param location: Location, Mouse location to select channel
    :type location: typing.List[float]
    :param extend: Extend, Extend selection rather than clearing the existing selection
    :type extend: bool
    '''

    pass


def dopesheet_view_all():
    ''' Reset viewable area to show full keyframe range

    '''

    pass


def filter_tracks(track_threshold: float = 5.0):
    ''' Filter tracks which has weirdly looking spikes in motion curves

    :param track_threshold: Track Threshold, Filter Threshold to select problematic tracks
    :type track_threshold: float
    '''

    pass


def frame_jump(position: typing.Union[str, int] = 'PATHSTART'):
    ''' Jump to special frame

    :param position: Position, Position to jump to * PATHSTART Path Start -- Jump to start of current path. * PATHEND Path End -- Jump to end of current path. * FAILEDPREV Previous Failed -- Jump to previous failed frame. * FAILNEXT Next Failed -- Jump to next failed frame.
    :type position: typing.Union[str, int]
    '''

    pass


def graph_center_current_frame():
    ''' Scroll view so current frame would be centered

    '''

    pass


def graph_delete_curve():
    ''' Delete track corresponding to the selected curve

    '''

    pass


def graph_delete_knot():
    ''' Delete curve knots

    '''

    pass


def graph_disable_markers(action: typing.Union[str, int] = 'DISABLE'):
    ''' Disable/enable selected markers

    :param action: Action, Disable action to execute * DISABLE Disable -- Disable selected markers. * ENABLE Enable -- Enable selected markers. * TOGGLE Toggle -- Toggle disabled flag for selected markers.
    :type action: typing.Union[str, int]
    '''

    pass


def graph_select(location: typing.List[float] = (0.0, 0.0),
                 extend: bool = False):
    ''' Select graph curves

    :param location: Location, Mouse location to select nearest entity
    :type location: typing.List[float]
    :param extend: Extend, Extend selection rather than clearing the existing selection
    :type extend: bool
    '''

    pass


def graph_select_all_markers(action: typing.Union[str, int] = 'TOGGLE'):
    ''' Change selection of all markers of active track

    :param action: Action, Selection action to execute * TOGGLE Toggle -- Toggle selection for all elements. * SELECT Select -- Select all elements. * DESELECT Deselect -- Deselect all elements. * INVERT Invert -- Invert selection of all elements.
    :type action: typing.Union[str, int]
    '''

    pass


def graph_select_box(xmin: int = 0,
                     xmax: int = 0,
                     ymin: int = 0,
                     ymax: int = 0,
                     wait_for_input: bool = True,
                     deselect: bool = False,
                     extend: bool = True):
    ''' Select curve points using box selection

    :param xmin: X Min
    :type xmin: int
    :param xmax: X Max
    :type xmax: int
    :param ymin: Y Min
    :type ymin: int
    :param ymax: Y Max
    :type ymax: int
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    :param deselect: Deselect, Deselect rather than select items
    :type deselect: bool
    :param extend: Extend, Extend selection instead of deselecting everything first
    :type extend: bool
    '''

    pass


def graph_view_all():
    ''' View all curves in editor

    '''

    pass


def hide_tracks(unselected: bool = False):
    ''' Hide selected tracks

    :param unselected: Unselected, Hide unselected tracks
    :type unselected: bool
    '''

    pass


def hide_tracks_clear():
    ''' Clear hide selected tracks

    '''

    pass


def join_tracks():
    ''' Join selected tracks

    '''

    pass


def keyframe_delete():
    ''' Delete a keyframe from selected tracks at current frame

    '''

    pass


def keyframe_insert():
    ''' Insert a keyframe to selected tracks at current frame

    '''

    pass


def lock_selection_toggle():
    ''' Toggle Lock Selection option of the current clip editor

    '''

    pass


def lock_tracks(action: typing.Union[str, int] = 'LOCK'):
    ''' Lock/unlock selected tracks

    :param action: Action, Lock action to execute * LOCK Lock -- Lock selected tracks. * UNLOCK Unlock -- Unlock selected tracks. * TOGGLE Toggle -- Toggle locked flag for selected tracks.
    :type action: typing.Union[str, int]
    '''

    pass


def mode_set(mode: typing.Union[str, int] = 'TRACKING'):
    ''' Set the clip interaction mode

    :param mode: Mode * TRACKING Tracking -- Show tracking and solving tools. * MASK Mask -- Show mask editing tools.
    :type mode: typing.Union[str, int]
    '''

    pass


def open(directory: str = "",
         files: typing.
         Union[typing.Dict[str, 'bpy.types.OperatorFileListElement'], typing.
               List['bpy.types.OperatorFileListElement'],
               'bpy_prop_collection'] = None,
         hide_props_region: bool = True,
         filter_blender: bool = False,
         filter_backup: bool = False,
         filter_image: bool = True,
         filter_movie: bool = True,
         filter_python: bool = False,
         filter_font: bool = False,
         filter_sound: bool = False,
         filter_text: bool = False,
         filter_archive: bool = False,
         filter_btx: bool = False,
         filter_collada: bool = False,
         filter_alembic: bool = False,
         filter_usd: bool = False,
         filter_obj: bool = False,
         filter_volume: bool = False,
         filter_folder: bool = True,
         filter_blenlib: bool = False,
         filemode: int = 9,
         relative_path: bool = True,
         show_multiview: bool = False,
         use_multiview: bool = False,
         display_type: typing.Union[str, int] = 'DEFAULT',
         sort_method: typing.Union[str, int] = ''):
    ''' Load a sequence of frames or a movie file

    :param directory: Directory, Directory of the file
    :type directory: str
    :param files: Files
    :type files: typing.Union[typing.Dict[str, 'bpy.types.OperatorFileListElement'], typing.List['bpy.types.OperatorFileListElement'], 'bpy_prop_collection']
    :param hide_props_region: Hide Operator Properties, Collapse the region displaying the operator settings
    :type hide_props_region: bool
    :param filter_blender: Filter .blend files
    :type filter_blender: bool
    :param filter_backup: Filter .blend files
    :type filter_backup: bool
    :param filter_image: Filter image files
    :type filter_image: bool
    :param filter_movie: Filter movie files
    :type filter_movie: bool
    :param filter_python: Filter python files
    :type filter_python: bool
    :param filter_font: Filter font files
    :type filter_font: bool
    :param filter_sound: Filter sound files
    :type filter_sound: bool
    :param filter_text: Filter text files
    :type filter_text: bool
    :param filter_archive: Filter archive files
    :type filter_archive: bool
    :param filter_btx: Filter btx files
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files
    :type filter_alembic: bool
    :param filter_usd: Filter USD files
    :type filter_usd: bool
    :param filter_obj: Filter OBJ files
    :type filter_obj: bool
    :param filter_volume: Filter OpenVDB volume files
    :type filter_volume: bool
    :param filter_folder: Filter folders
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file
    :type relative_path: bool
    :param show_multiview: Enable Multi-View
    :type show_multiview: bool
    :param use_multiview: Use Multi-View
    :type use_multiview: bool
    :param display_type: Display Type * DEFAULT Default -- Automatically determine display type for files. * LIST_VERTICAL Short List -- Display files as short list. * LIST_HORIZONTAL Long List -- Display files as a detailed list. * THUMBNAIL Thumbnails -- Display files as thumbnails.
    :type display_type: typing.Union[str, int]
    :param sort_method: File sorting mode
    :type sort_method: typing.Union[str, int]
    '''

    pass


def paste_tracks():
    ''' Paste tracks from clipboard

    '''

    pass


def prefetch():
    ''' Prefetch frames from disk for faster playback/tracking

    '''

    pass


def rebuild_proxy():
    ''' Rebuild all selected proxies and timecode indices in the background

    '''

    pass


def refine_markers(backwards: bool = False):
    ''' Refine selected markers positions by running the tracker from track's reference to current frame

    :param backwards: Backwards, Do backwards tracking
    :type backwards: bool
    '''

    pass


def reload():
    ''' Reload clip

    '''

    pass


def select(extend: bool = False,
           deselect_all: bool = False,
           location: typing.List[float] = (0.0, 0.0)):
    ''' Select tracking markers

    :param extend: Extend, Extend selection rather than clearing the existing selection
    :type extend: bool
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor
    :type deselect_all: bool
    :param location: Location, Mouse location in normalized coordinates, 0.0 to 1.0 is within the image bounds
    :type location: typing.List[float]
    '''

    pass


def select_all(action: typing.Union[str, int] = 'TOGGLE'):
    ''' Change selection of all tracking markers

    :param action: Action, Selection action to execute * TOGGLE Toggle -- Toggle selection for all elements. * SELECT Select -- Select all elements. * DESELECT Deselect -- Deselect all elements. * INVERT Invert -- Invert selection of all elements.
    :type action: typing.Union[str, int]
    '''

    pass


def select_box(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[str, int] = 'SET'):
    ''' Select markers using box selection

    :param xmin: X Min
    :type xmin: int
    :param xmax: X Max
    :type xmax: int
    :param ymin: Y Min
    :type ymin: int
    :param ymax: Y Max
    :type ymax: int
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    :param mode: Mode * SET Set -- Set a new selection. * ADD Extend -- Extend existing selection. * SUB Subtract -- Subtract existing selection.
    :type mode: typing.Union[str, int]
    '''

    pass


def select_circle(x: int = 0,
                  y: int = 0,
                  radius: int = 25,
                  wait_for_input: bool = True,
                  mode: typing.Union[str, int] = 'SET'):
    ''' Select markers using circle selection

    :param x: X
    :type x: int
    :param y: Y
    :type y: int
    :param radius: Radius
    :type radius: int
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    :param mode: Mode * SET Set -- Set a new selection. * ADD Extend -- Extend existing selection. * SUB Subtract -- Subtract existing selection.
    :type mode: typing.Union[str, int]
    '''

    pass


def select_grouped(group: typing.Union[str, int] = 'ESTIMATED'):
    ''' Select all tracks from specified group

    :param group: Action, Clear action to execute * KEYFRAMED Keyframed Tracks -- Select all keyframed tracks. * ESTIMATED Estimated Tracks -- Select all estimated tracks. * TRACKED Tracked Tracks -- Select all tracked tracks. * LOCKED Locked Tracks -- Select all locked tracks. * DISABLED Disabled Tracks -- Select all disabled tracks. * COLOR Tracks with Same Color -- Select all tracks with same color as active track. * FAILED Failed Tracks -- Select all tracks which failed to be reconstructed.
    :type group: typing.Union[str, int]
    '''

    pass


def select_lasso(path: typing.Union[
        typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.
        List['bpy.types.OperatorMousePath'], 'bpy_prop_collection'] = None,
                 mode: typing.Union[str, int] = 'SET'):
    ''' Select markers using lasso selection

    :param path: Path
    :type path: typing.Union[typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.List['bpy.types.OperatorMousePath'], 'bpy_prop_collection']
    :param mode: Mode * SET Set -- Set a new selection. * ADD Extend -- Extend existing selection. * SUB Subtract -- Subtract existing selection.
    :type mode: typing.Union[str, int]
    '''

    pass


def set_active_clip():
    ''' Undocumented, consider contributing <https://developer.blender.org/T51061> __. :file: startup/bl_operators/clip.py\:211 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/clip.py$211> _

    '''

    pass


def set_axis(axis: typing.Union[str, int] = 'X'):
    ''' Set direction of scene axis rotating camera (or its parent if present) and assume selected track lies on real axis, joining it with the origin

    :param axis: Axis, Axis to use to align bundle along * X X -- Align bundle align X axis. * Y Y -- Align bundle align Y axis.
    :type axis: typing.Union[str, int]
    '''

    pass


def set_center_principal():
    ''' Set optical center to center of footage

    '''

    pass


def set_origin(use_median: bool = False):
    ''' Set active marker as origin by moving camera (or its parent if present) in 3D space

    :param use_median: Use Median, Set origin to median point of selected bundles
    :type use_median: bool
    '''

    pass


def set_plane(plane: typing.Union[str, int] = 'FLOOR'):
    ''' Set plane based on 3 selected bundles by moving camera (or its parent if present) in 3D space

    :param plane: Plane, Plane to be used for orientation * FLOOR Floor -- Set floor plane. * WALL Wall -- Set wall plane.
    :type plane: typing.Union[str, int]
    '''

    pass


def set_scale(distance: float = 0.0):
    ''' Set scale of scene by scaling camera (or its parent if present)

    :param distance: Distance, Distance between selected tracks
    :type distance: float
    '''

    pass


def set_scene_frames():
    ''' Set scene's start and end frame to match clip's start frame and length

    '''

    pass


def set_solution_scale(distance: float = 0.0):
    ''' Set object solution scale using distance between two selected tracks

    :param distance: Distance, Distance between selected tracks
    :type distance: float
    '''

    pass


def set_solver_keyframe(keyframe: typing.Union[str, int] = 'KEYFRAME_A'):
    ''' Set keyframe used by solver

    :param keyframe: Keyframe, Keyframe to set
    :type keyframe: typing.Union[str, int]
    '''

    pass


def set_viewport_background():
    ''' Set current movie clip as a camera background in 3D Viewport (works only when a 3D Viewport is visible) :file: startup/bl_operators/clip.py\:410 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/clip.py$410> _

    '''

    pass


def setup_tracking_scene():
    ''' Prepare scene for compositing 3D objects into this footage :file: startup/bl_operators/clip.py\:972 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/clip.py$972> _

    '''

    pass


def slide_marker(offset: typing.List[float] = (0.0, 0.0)):
    ''' Slide marker areas

    :param offset: Offset, Offset in floating-point units, 1.0 is the width and height of the image
    :type offset: typing.List[float]
    '''

    pass


def slide_plane_marker():
    ''' Slide plane marker areas

    '''

    pass


def solve_camera():
    ''' Solve camera motion from tracks

    '''

    pass


def stabilize_2d_add():
    ''' Add selected tracks to 2D translation stabilization

    '''

    pass


def stabilize_2d_remove():
    ''' Remove selected track from translation stabilization

    '''

    pass


def stabilize_2d_rotation_add():
    ''' Add selected tracks to 2D rotation stabilization

    '''

    pass


def stabilize_2d_rotation_remove():
    ''' Remove selected track from rotation stabilization

    '''

    pass


def stabilize_2d_rotation_select():
    ''' Select tracks which are used for rotation stabilization

    '''

    pass


def stabilize_2d_select():
    ''' Select tracks which are used for translation stabilization

    '''

    pass


def track_color_preset_add(name: str = "",
                           remove_name: bool = False,
                           remove_active: bool = False):
    ''' Add or remove a Clip Track Color Preset

    :param name: Name, Name of the preset, used to make the path name
    :type name: str
    :param remove_name: remove_name
    :type remove_name: bool
    :param remove_active: remove_active
    :type remove_active: bool
    '''

    pass


def track_copy_color():
    ''' Copy color to all selected tracks

    '''

    pass


def track_markers(backwards: bool = False, sequence: bool = False):
    ''' Track selected markers

    :param backwards: Backwards, Do backwards tracking
    :type backwards: bool
    :param sequence: Track Sequence, Track marker during image sequence rather than single image
    :type sequence: bool
    '''

    pass


def track_settings_as_default():
    ''' Copy tracking settings from active track to default settings :file: startup/bl_operators/clip.py\:1001 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/clip.py$1001> _

    '''

    pass


def track_settings_to_track():
    ''' Copy tracking settings from active track to selected tracks :file: startup/bl_operators/clip.py\:1050 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/clip.py$1050> _

    '''

    pass


def track_to_empty():
    ''' Create an Empty object which will be copying movement of active track :file: startup/bl_operators/clip.py\:258 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/clip.py$258> _

    '''

    pass


def tracking_object_new():
    ''' Add new object for tracking

    '''

    pass


def tracking_object_remove():
    ''' Remove object for tracking

    '''

    pass


def tracking_settings_preset_add(name: str = "",
                                 remove_name: bool = False,
                                 remove_active: bool = False):
    ''' Add or remove a motion tracking settings preset

    :param name: Name, Name of the preset, used to make the path name
    :type name: str
    :param remove_name: remove_name
    :type remove_name: bool
    :param remove_active: remove_active
    :type remove_active: bool
    '''

    pass


def view_all(fit_view: bool = False):
    ''' View whole image with markers

    :param fit_view: Fit View, Fit frame to the viewport
    :type fit_view: bool
    '''

    pass


def view_center_cursor():
    ''' Center the view so that the cursor is in the middle of the view

    '''

    pass


def view_ndof():
    ''' Use a 3D mouse device to pan/zoom the view

    '''

    pass


def view_pan(offset: typing.List[float] = (0.0, 0.0)):
    ''' Pan the view

    :param offset: Offset, Offset in floating-point units, 1.0 is the width and height of the image
    :type offset: typing.List[float]
    '''

    pass


def view_selected():
    ''' View all selected elements

    '''

    pass


def view_zoom(factor: float = 0.0, use_cursor_init: bool = True):
    ''' Zoom in/out the view

    :param factor: Factor, Zoom factor, values higher than 1.0 zoom in, lower values zoom out
    :type factor: float
    :param use_cursor_init: Use Mouse Position, Allow the initial mouse position to be used
    :type use_cursor_init: bool
    '''

    pass


def view_zoom_in(location: typing.List[float] = (0.0, 0.0)):
    ''' Zoom in the view

    :param location: Location, Cursor location in screen coordinates
    :type location: typing.List[float]
    '''

    pass


def view_zoom_out(location: typing.List[float] = (0.0, 0.0)):
    ''' Zoom out the view

    :param location: Location, Cursor location in normalized (0.0 to 1.0) coordinates
    :type location: typing.List[float]
    '''

    pass


def view_zoom_ratio(ratio: float = 0.0):
    ''' Set the zoom ratio (based on clip size)

    :param ratio: Ratio, Zoom ratio, 1.0 is 1:1, higher is zoomed in, lower is zoomed out
    :type ratio: float
    '''

    pass



import sys
import typing


def add_target():
    ''' Add a target to the constraint :file: startup/bl_operators/constraint.py\:25 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/constraint.py$25> _

    '''

    pass


def apply(constraint: str = "",
          owner: typing.Union[str, int] = 'OBJECT',
          report: bool = False):
    ''' Apply constraint and remove from the stack

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    :param report: Report, Create a notification after the operation
    :type report: bool
    '''

    pass


def childof_clear_inverse(constraint: str = "",
                          owner: typing.Union[str, int] = 'OBJECT'):
    ''' Clear inverse correction for Child Of constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def childof_set_inverse(constraint: str = "",
                        owner: typing.Union[str, int] = 'OBJECT'):
    ''' Set inverse correction for Child Of constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def copy(constraint: str = "",
         owner: typing.Union[str, int] = 'OBJECT',
         report: bool = False):
    ''' Duplicate constraint at the same position in the stack

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    :param report: Report, Create a notification after the operation
    :type report: bool
    '''

    pass


def copy_to_selected(constraint: str = "",
                     owner: typing.Union[str, int] = 'OBJECT'):
    ''' Copy constraint to other selected objects/bones

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def delete(constraint: str = "",
           owner: typing.Union[str, int] = 'OBJECT',
           report: bool = False):
    ''' Remove constraint from constraint stack

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    :param report: Report, Create a notification after the operation
    :type report: bool
    '''

    pass


def disable_keep_transform():
    ''' Set the influence of this constraint to zero while trying to maintain the object's transformation. Other active constraints can still influence the final transformation :file: startup/bl_operators/constraint.py\:85 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/constraint.py$85> _

    '''

    pass


def followpath_path_animate(constraint: str = "",
                            owner: typing.Union[str, int] = 'OBJECT',
                            frame_start: int = 1,
                            length: int = 100):
    ''' Add default animation for path used by constraint if it isn't animated already

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    :param frame_start: Start Frame, First frame of path animation
    :type frame_start: int
    :param length: Length, Number of frames that path animation should take
    :type length: int
    '''

    pass


def limitdistance_reset(constraint: str = "",
                        owner: typing.Union[str, int] = 'OBJECT'):
    ''' Reset limiting distance for Limit Distance Constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def move_down(constraint: str = "", owner: typing.Union[str, int] = 'OBJECT'):
    ''' Move constraint down in constraint stack

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def move_to_index(constraint: str = "",
                  owner: typing.Union[str, int] = 'OBJECT',
                  index: int = 0):
    ''' Change the constraint's position in the list so it evaluates after the set number of others

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    :param index: Index, The index to move the constraint to
    :type index: int
    '''

    pass


def move_up(constraint: str = "", owner: typing.Union[str, int] = 'OBJECT'):
    ''' Move constraint up in constraint stack

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def normalize_target_weights():
    ''' Normalize weights of all target bones :file: startup/bl_operators/constraint.py\:60 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/constraint.py$60> _

    '''

    pass


def objectsolver_clear_inverse(constraint: str = "",
                               owner: typing.Union[str, int] = 'OBJECT'):
    ''' Clear inverse correction for Object Solver constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def objectsolver_set_inverse(constraint: str = "",
                             owner: typing.Union[str, int] = 'OBJECT'):
    ''' Set inverse correction for Object Solver constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def remove_target(index: int = 0):
    ''' Remove the target from the constraint

    :param index: index
    :type index: int
    '''

    pass


def stretchto_reset(constraint: str = "",
                    owner: typing.Union[str, int] = 'OBJECT'):
    ''' Reset original length of bone for Stretch To Constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object -- Edit a constraint on the active object. * BONE Bone -- Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass



import sys
import typing


def denoise_animation(input_filepath: str = "", output_filepath: str = ""):
    ''' Denoise rendered animation sequence using current scene and view layer settings. Requires denoising data passes and output to OpenEXR multilayer files

    :param input_filepath: Input Filepath, File path for image to denoise. If not specified, uses the render file path and frame range from the scene
    :type input_filepath: str
    :param output_filepath: Output Filepath, If not specified, renders will be denoised in-place
    :type output_filepath: str
    '''

    pass


def merge_images(input_filepath1: str = "",
                 input_filepath2: str = "",
                 output_filepath: str = ""):
    ''' Combine OpenEXR multilayer images rendered with different sample ranges into one image with reduced noise

    :param input_filepath1: Input Filepath, File path for image to merge
    :type input_filepath1: str
    :param input_filepath2: Input Filepath, File path for image to merge
    :type input_filepath2: str
    :param output_filepath: Output Filepath, File path for merged image
    :type output_filepath: str
    '''

    pass


def use_shading_nodes():
    ''' Enable nodes on a material, world or light :file: addons/cycles/operators.py\:24 <https://developer.blender.org/diffusion/BA/addons/cycles/operators.py$24> _

    '''

    pass



import sys
import typing
import bpy.types


def active_frame_delete():
    ''' Delete the active frame for the active Grease Pencil Layer

    '''

    pass


def active_frames_delete_all():
    ''' Delete the active frame(s) of all editable Grease Pencil layers

    '''

    pass


def annotate(mode: typing.Union[str, int] = 'DRAW',
             arrowstyle_start: typing.Union[str, int] = 'NONE',
             arrowstyle_end: typing.Union[str, int] = 'NONE',
             use_stabilizer: bool = False,
             stabilizer_factor: float = 0.75,
             stabilizer_radius: int = 35,
             stroke: typing.
             Union[typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.
                   List['bpy.types.OperatorStrokeElement'],
                   'bpy_prop_collection'] = None,
             wait_for_input: bool = True):
    ''' Make annotations on the active data

    :param mode: Mode, Way to interpret mouse movements * DRAW Draw Freehand -- Draw freehand stroke(s). * DRAW_STRAIGHT Draw Straight Lines -- Draw straight line segment(s). * DRAW_POLY Draw Poly Line -- Click to place endpoints of straight line segments (connected). * ERASER Eraser -- Erase Annotation strokes.
    :type mode: typing.Union[str, int]
    :param arrowstyle_start: Start Arrow Style, Stroke start style * NONE None -- Don't use any arrow/style in corner. * ARROW Arrow -- Use closed arrow style. * ARROW_OPEN Open Arrow -- Use open arrow style. * ARROW_OPEN_INVERTED Segment -- Use perpendicular segment style. * DIAMOND Square -- Use square style.
    :type arrowstyle_start: typing.Union[str, int]
    :param arrowstyle_end: End Arrow Style, Stroke end style * NONE None -- Don't use any arrow/style in corner. * ARROW Arrow -- Use closed arrow style. * ARROW_OPEN Open Arrow -- Use open arrow style. * ARROW_OPEN_INVERTED Segment -- Use perpendicular segment style. * DIAMOND Square -- Use square style.
    :type arrowstyle_end: typing.Union[str, int]
    :param use_stabilizer: Stabilize Stroke, Helper to draw smooth and clean lines. Press Shift for an invert effect (even if this option is not active)
    :type use_stabilizer: bool
    :param stabilizer_factor: Stabilizer Stroke Factor, Higher values gives a smoother stroke
    :type stabilizer_factor: float
    :param stabilizer_radius: Stabilizer Stroke Radius, Minimum distance from last point before stroke continues
    :type stabilizer_radius: int
    :param stroke: Stroke
    :type stroke: typing.Union[typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.List['bpy.types.OperatorStrokeElement'], 'bpy_prop_collection']
    :param wait_for_input: Wait for Input, Wait for first click instead of painting immediately
    :type wait_for_input: bool
    '''

    pass


def annotation_active_frame_delete():
    ''' Delete the active frame for the active Annotation Layer

    '''

    pass


def annotation_add():
    ''' Add new Annotation data-block

    '''

    pass


def bake_grease_pencil_animation(
        frame_start: int = 1,
        frame_end: int = 250,
        step: int = 1,
        only_selected: bool = False,
        frame_target: int = 1,
        project_type: typing.Union[str, int] = 'KEEP'):
    ''' Bake grease pencil object transform to grease pencil keyframes

    :param frame_start: Start Frame, The start frame
    :type frame_start: int
    :param frame_end: End Frame, The end frame of animation
    :type frame_end: int
    :param step: Step, Step between generated frames
    :type step: int
    :param only_selected: Only Selected Keyframes, Convert only selected keyframes
    :type only_selected: bool
    :param frame_target: Target Frame, Destination frame
    :type frame_target: int
    :param project_type: Projection Type * KEEP No Reproject. * FRONT Front -- Reproject the strokes using the X-Z plane. * SIDE Side -- Reproject the strokes using the Y-Z plane. * TOP Top -- Reproject the strokes using the X-Y plane. * VIEW View -- Reproject the strokes to end up on the same plane, as if drawn from the current viewpoint using 'Cursor' Stroke Placement. * CURSOR Cursor -- Reproject the strokes using the orientation of 3D cursor.
    :type project_type: typing.Union[str, int]
    '''

    pass


def bake_mesh_animation(target: typing.Union[str, int] = 'NEW',
                        frame_start: int = 1,
                        frame_end: int = 250,
                        step: int = 1,
                        thickness: int = 1,
                        angle: float = 1.22173,
                        offset: float = 0.001,
                        seams: bool = False,
                        faces: bool = True,
                        only_selected: bool = False,
                        frame_target: int = 1,
                        project_type: typing.Union[str, int] = 'VIEW'):
    ''' Bake mesh animation to grease pencil strokes

    :param target: Target Object, Target grease pencil
    :type target: typing.Union[str, int]
    :param frame_start: Start Frame, The start frame
    :type frame_start: int
    :param frame_end: End Frame, The end frame of animation
    :type frame_end: int
    :param step: Step, Step between generated frames
    :type step: int
    :param thickness: Thickness
    :type thickness: int
    :param angle: Threshold Angle, Threshold to determine ends of the strokes
    :type angle: float
    :param offset: Stroke Offset, Offset strokes from fill
    :type offset: float
    :param seams: Only Seam Edges, Convert only seam edges
    :type seams: bool
    :param faces: Export Faces, Export faces as filled strokes
    :type faces: bool
    :param only_selected: Only Selected Keyframes, Convert only selected keyframes
    :type only_selected: bool
    :param frame_target: Target Frame, Destination frame
    :type frame_target: int
    :param project_type: Projection Type * KEEP No Reproject. * FRONT Front -- Reproject the strokes using the X-Z plane. * SIDE Side -- Reproject the strokes using the Y-Z plane. * TOP Top -- Reproject the strokes using the X-Y plane. * VIEW View -- Reproject the strokes to end up on the same plane, as if drawn from the current viewpoint using 'Cursor' Stroke Placement. * CURSOR Cursor -- Reproject the strokes using the orientation of 3D cursor.
    :type project_type: typing.Union[str, int]
    '''

    pass


def blank_frame_add(all_layers: bool = False):
    ''' Insert a blank frame on the current frame (all subsequently existing frames, if any, are shifted right by one frame)

    :param all_layers: All Layers, Create blank frame in all layers, not only active
    :type all_layers: bool
    '''

    pass


def brush_reset():
    ''' Reset brush to default parameters

    '''

    pass


def brush_reset_all():
    ''' Delete all mode brushes and recreate a default set

    '''

    pass


def convert(type: typing.Union[str, int] = 'PATH',
            bevel_depth: float = 0.0,
            bevel_resolution: int = 0,
            use_normalize_weights: bool = True,
            radius_multiplier: float = 1.0,
            use_link_strokes: bool = False,
            timing_mode: typing.Union[str, int] = 'FULL',
            frame_range: int = 100,
            start_frame: int = 1,
            use_realtime: bool = False,
            end_frame: int = 250,
            gap_duration: float = 0.0,
            gap_randomness: float = 0.0,
            seed: int = 0,
            use_timing_data: bool = False):
    ''' Convert the active Grease Pencil layer to a new Curve Object

    :param type: Type, Which type of curve to convert to * PATH Path -- Animation path. * CURVE Bezier Curve -- Smooth Bezier curve. * POLY Polygon Curve -- Bezier curve with straight-line segments (vector handles).
    :type type: typing.Union[str, int]
    :param bevel_depth: Bevel Depth
    :type bevel_depth: float
    :param bevel_resolution: Bevel Resolution, Bevel resolution when depth is non-zero
    :type bevel_resolution: int
    :param use_normalize_weights: Normalize Weight, Normalize weight (set from stroke width)
    :type use_normalize_weights: bool
    :param radius_multiplier: Radius Factor, Multiplier for the points' radii (set from stroke width)
    :type radius_multiplier: float
    :param use_link_strokes: Link Strokes, Whether to link strokes with zero-radius sections of curves
    :type use_link_strokes: bool
    :param timing_mode: Timing Mode, How to use timing data stored in strokes * NONE No Timing -- Ignore timing. * LINEAR Linear -- Simple linear timing. * FULL Original -- Use the original timing, gaps included. * CUSTOMGAP Custom Gaps -- Use the original timing, but with custom gap lengths (in frames).
    :type timing_mode: typing.Union[str, int]
    :param frame_range: Frame Range, The duration of evaluation of the path control curve
    :type frame_range: int
    :param start_frame: Start Frame, The start frame of the path control curve
    :type start_frame: int
    :param use_realtime: Realtime, Whether the path control curve reproduces the drawing in realtime, starting from Start Frame
    :type use_realtime: bool
    :param end_frame: End Frame, The end frame of the path control curve (if Realtime is not set)
    :type end_frame: int
    :param gap_duration: Gap Duration, Custom Gap mode: (Average) length of gaps, in frames (Note: Realtime value, will be scaled if Realtime is not set)
    :type gap_duration: float
    :param gap_randomness: Gap Randomness, Custom Gap mode: Number of frames that gap lengths can vary
    :type gap_randomness: float
    :param seed: Random Seed, Custom Gap mode: Random generator seed
    :type seed: int
    :param use_timing_data: Has Valid Timing, Whether the converted Grease Pencil layer has valid timing data (internal use)
    :type use_timing_data: bool
    '''

    pass


def convert_old_files(annotation: bool = False):
    ''' Convert 2.7x grease pencil files to 2.80

    :param annotation: Annotation, Convert to Annotations
    :type annotation: bool
    '''

    pass


def copy():
    ''' Copy selected Grease Pencil points and strokes

    '''

    pass


def data_unlink():
    ''' Unlink active Annotation data-block

    '''

    pass


def delete(type: typing.Union[str, int] = 'POINTS'):
    ''' Delete selected Grease Pencil strokes, vertices, or frames

    :param type: Type, Method used for deleting Grease Pencil data * POINTS Points -- Delete selected points and split strokes into segments. * STROKES Strokes -- Delete selected strokes. * FRAME Frame -- Delete active frame.
    :type type: typing.Union[str, int]
    '''

    pass


def dissolve(type: typing.Union[str, int] = 'POINTS'):
    ''' Delete selected points without splitting strokes

    :param type: Type, Method used for dissolving stroke points * POINTS Dissolve -- Dissolve selected points. * BETWEEN Dissolve Between -- Dissolve points between selected points. * UNSELECT Dissolve Unselect -- Dissolve all unselected points.
    :type type: typing.Union[str, int]
    '''

    pass


def draw(mode: typing.Union[str, int] = 'DRAW',
         stroke: typing.
         Union[typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.
               List['bpy.types.OperatorStrokeElement'],
               'bpy_prop_collection'] = None,
         wait_for_input: bool = True,
         disable_straight: bool = False,
         disable_fill: bool = False,
         disable_stabilizer: bool = False,
         guide_last_angle: float = 0.0):
    ''' Draw a new stroke in the active Grease Pencil object

    :param mode: Mode, Way to interpret mouse movements * DRAW Draw Freehand -- Draw freehand stroke(s). * DRAW_STRAIGHT Draw Straight Lines -- Draw straight line segment(s). * ERASER Eraser -- Erase Grease Pencil strokes.
    :type mode: typing.Union[str, int]
    :param stroke: Stroke
    :type stroke: typing.Union[typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.List['bpy.types.OperatorStrokeElement'], 'bpy_prop_collection']
    :param wait_for_input: Wait for Input, Wait for first click instead of painting immediately
    :type wait_for_input: bool
    :param disable_straight: No Straight lines, Disable key for straight lines
    :type disable_straight: bool
    :param disable_fill: No Fill Areas, Disable fill to use stroke as fill boundary
    :type disable_fill: bool
    :param disable_stabilizer: No Stabilizer
    :type disable_stabilizer: bool
    :param guide_last_angle: Angle, Speed guide angle
    :type guide_last_angle: float
    '''

    pass


def duplicate():
    ''' Duplicate the selected Grease Pencil strokes

    '''

    pass


def duplicate_move(GPENCIL_OT_duplicate=None, TRANSFORM_OT_translate=None):
    ''' Make copies of the selected Grease Pencil strokes and move them

    :param GPENCIL_OT_duplicate: Duplicate Strokes, Duplicate the selected Grease Pencil strokes
    :param TRANSFORM_OT_translate: Move, Move selected items
    '''

    pass


def editmode_toggle(back: bool = False):
    ''' Enter/Exit edit mode for Grease Pencil strokes

    :param back: Return to Previous Mode, Return to previous mode
    :type back: bool
    '''

    pass


def extract_palette_vertex(selected: bool = False, threshold: int = 1):
    ''' Extract all colors used in Grease Pencil Vertex and create a Palette

    :param selected: Only Selected, Convert only selected strokes
    :type selected: bool
    :param threshold: Threshold
    :type threshold: int
    '''

    pass


def extrude():
    ''' Extrude the selected Grease Pencil points

    '''

    pass


def extrude_move(GPENCIL_OT_extrude=None, TRANSFORM_OT_translate=None):
    ''' Extrude selected points and move them

    :param GPENCIL_OT_extrude: Extrude Stroke Points, Extrude the selected Grease Pencil points
    :param TRANSFORM_OT_translate: Move, Move selected items
    '''

    pass


def fill(on_back: bool = False):
    ''' Fill with color the shape formed by strokes

    :param on_back: Draw on Back, Send new stroke to back
    :type on_back: bool
    '''

    pass


def frame_clean_duplicate(type: typing.Union[str, int] = 'ALL'):
    ''' Remove any duplicated frame

    :param type: Type
    :type type: typing.Union[str, int]
    '''

    pass


def frame_clean_fill(mode: typing.Union[str, int] = 'ACTIVE'):
    ''' Remove 'no fill' boundary strokes

    :param mode: Mode * ACTIVE Active Frame Only -- Clean active frame only. * ALL All Frames -- Clean all frames in all layers.
    :type mode: typing.Union[str, int]
    '''

    pass


def frame_clean_loose(limit: int = 1):
    ''' Remove loose points

    :param limit: Limit, Number of points to consider stroke as loose
    :type limit: int
    '''

    pass


def frame_duplicate(mode: typing.Union[str, int] = 'ACTIVE'):
    ''' Make a copy of the active Grease Pencil Frame

    :param mode: Mode * ACTIVE Active -- Duplicate frame in active layer only. * ALL All -- Duplicate active frames in all layers.
    :type mode: typing.Union[str, int]
    '''

    pass


def generate_weights(mode: typing.Union[str, int] = 'NAME',
                     armature: typing.Union[str, int] = 'DEFAULT',
                     ratio: float = 0.1,
                     decay: float = 0.8):
    ''' Generate automatic weights for armatures (requires armature modifier)

    :param mode: Mode
    :type mode: typing.Union[str, int]
    :param armature: Armature, Armature to use
    :type armature: typing.Union[str, int]
    :param ratio: Ratio, Ratio between bone length and influence radius
    :type ratio: float
    :param decay: Decay, Factor to reduce influence depending of distance to bone axis
    :type decay: float
    '''

    pass


def guide_rotate(increment: bool = True, angle: float = 0.0):
    ''' Rotate guide angle

    :param increment: Increment, Increment angle
    :type increment: bool
    :param angle: Angle, Guide angle
    :type angle: float
    '''

    pass


def hide(unselected: bool = False):
    ''' Hide selected/unselected Grease Pencil layers

    :param unselected: Unselected, Hide unselected rather than selected layers
    :type unselected: bool
    '''

    pass


def image_to_grease_pencil(size: float = 0.005, mask: bool = False):
    ''' Generate a Grease Pencil Object using Image as source

    :param size: Point Size, Size used for grease pencil points
    :type size: float
    :param mask: Generate Mask, Create an inverted image for masking using alpha channel
    :type mask: bool
    '''

    pass


def interpolate(shift: float = 0.0,
                layers: typing.Union[str, int] = 'ACTIVE',
                interpolate_selected_only: bool = False,
                flip: typing.Union[str, int] = 'AUTO',
                smooth_steps: int = 1,
                smooth_factor: float = 0.0,
                release_confirm: bool = False):
    ''' Interpolate grease pencil strokes between frames

    :param shift: Shift, Bias factor for which frame has more influence on the interpolated strokes
    :type shift: float
    :param layers: Layer, Layers included in the interpolation
    :type layers: typing.Union[str, int]
    :param interpolate_selected_only: Only Selected, Interpolate only selected strokes
    :type interpolate_selected_only: bool
    :param flip: Flip Mode, Invert destination stroke to match start and end with source stroke
    :type flip: typing.Union[str, int]
    :param smooth_steps: Iterations, Number of times to smooth newly created strokes
    :type smooth_steps: int
    :param smooth_factor: Smooth, Amount of smoothing to apply to interpolated strokes, to reduce jitter/noise
    :type smooth_factor: float
    :param release_confirm: Confirm on Release
    :type release_confirm: bool
    '''

    pass


def interpolate_reverse():
    ''' Remove breakdown frames generated by interpolating between two Grease Pencil frames

    '''

    pass


def interpolate_sequence(step: int = 1,
                         layers: typing.Union[str, int] = 'ACTIVE',
                         interpolate_selected_only: bool = False,
                         flip: typing.Union[str, int] = 'AUTO',
                         smooth_steps: int = 1,
                         smooth_factor: float = 0.0,
                         type: typing.Union[str, int] = 'LINEAR',
                         easing: typing.Union[str, int] = 'AUTO',
                         back: float = 1.702,
                         amplitude: float = 0.15,
                         period: float = 0.15):
    ''' Generate 'in-betweens' to smoothly interpolate between Grease Pencil frames

    :param step: Step, Number of frames between generated interpolated frames
    :type step: int
    :param layers: Layer, Layers included in the interpolation
    :type layers: typing.Union[str, int]
    :param interpolate_selected_only: Only Selected, Interpolate only selected strokes
    :type interpolate_selected_only: bool
    :param flip: Flip Mode, Invert destination stroke to match start and end with source stroke
    :type flip: typing.Union[str, int]
    :param smooth_steps: Iterations, Number of times to smooth newly created strokes
    :type smooth_steps: int
    :param smooth_factor: Smooth, Amount of smoothing to apply to interpolated strokes, to reduce jitter/noise
    :type smooth_factor: float
    :param type: Type, Interpolation method to use the next time 'Interpolate Sequence' is run * LINEAR Linear -- Straight-line interpolation between A and B (i.e. no ease in/out). * CUSTOM Custom -- Custom interpolation defined using a curve map. * SINE Sinusoidal -- Sinusoidal easing (weakest, almost linear but with a slight curvature). * QUAD Quadratic -- Quadratic easing. * CUBIC Cubic -- Cubic easing. * QUART Quartic -- Quartic easing. * QUINT Quintic -- Quintic easing. * EXPO Exponential -- Exponential easing (dramatic). * CIRC Circular -- Circular easing (strongest and most dynamic). * BACK Back -- Cubic easing with overshoot and settle. * BOUNCE Bounce -- Exponentially decaying parabolic bounce, like when objects collide. * ELASTIC Elastic -- Exponentially decaying sine wave, like an elastic band.
    :type type: typing.Union[str, int]
    :param easing: Easing, Which ends of the segment between the preceding and following grease pencil frames easing interpolation is applied to * AUTO Automatic Easing -- Easing type is chosen automatically based on what the type of interpolation used (e.g. 'Ease In' for transitional types, and 'Ease Out' for dynamic effects). * EASE_IN Ease In -- Only on the end closest to the next keyframe. * EASE_OUT Ease Out -- Only on the end closest to the first keyframe. * EASE_IN_OUT Ease In and Out -- Segment between both keyframes.
    :type easing: typing.Union[str, int]
    :param back: Back, Amount of overshoot for 'back' easing
    :type back: float
    :param amplitude: Amplitude, Amount to boost elastic bounces for 'elastic' easing
    :type amplitude: float
    :param period: Period, Time between bounces for elastic easing
    :type period: float
    '''

    pass


def layer_active(layer: int = 0):
    ''' Active Grease Pencil layer

    :param layer: Grease Pencil Layer
    :type layer: int
    '''

    pass


def layer_add():
    ''' Add new layer or note for the active data-block

    '''

    pass


def layer_annotation_add():
    ''' Add new Annotation layer or note for the active data-block

    '''

    pass


def layer_annotation_move(type: typing.Union[str, int] = 'UP'):
    ''' Move the active Annotation layer up/down in the list

    :param type: Type
    :type type: typing.Union[str, int]
    '''

    pass


def layer_annotation_remove():
    ''' Remove active Annotation layer

    '''

    pass


def layer_change(layer: typing.Union[str, int] = 'DEFAULT'):
    ''' Change active Grease Pencil layer

    :param layer: Grease Pencil Layer
    :type layer: typing.Union[str, int]
    '''

    pass


def layer_duplicate(mode: typing.Union[str, int] = 'ALL'):
    ''' Make a copy of the active Grease Pencil layer

    :param mode: Mode
    :type mode: typing.Union[str, int]
    '''

    pass


def layer_duplicate_object(mode: typing.Union[str, int] = 'ALL',
                           only_active: bool = True):
    ''' Make a copy of the active Grease Pencil layer to selected object

    :param mode: Mode
    :type mode: typing.Union[str, int]
    :param only_active: Only Active, Copy only active Layer, uncheck to append all layers
    :type only_active: bool
    '''

    pass


def layer_isolate(affect_visibility: bool = False):
    ''' Toggle whether the active layer is the only one that can be edited and/or visible

    :param affect_visibility: Affect Visibility, In addition to toggling the editability, also affect the visibility
    :type affect_visibility: bool
    '''

    pass


def layer_mask_add(name: str = ""):
    ''' Add new layer as masking

    :param name: Layer, Name of the layer
    :type name: str
    '''

    pass


def layer_mask_move(type: typing.Union[str, int] = 'UP'):
    ''' Move the active Grease Pencil mask layer up/down in the list

    :param type: Type
    :type type: typing.Union[str, int]
    '''

    pass


def layer_mask_remove():
    ''' Remove Layer Mask

    '''

    pass


def layer_merge(mode: typing.Union[str, int] = 'ACTIVE'):
    ''' Combine Layers

    :param mode: Mode * ACTIVE Active -- Combine active layer into the layer below. * ALL All -- Combine all layers into the active layer.
    :type mode: typing.Union[str, int]
    '''

    pass


def layer_move(type: typing.Union[str, int] = 'UP'):
    ''' Move the active Grease Pencil layer up/down in the list

    :param type: Type
    :type type: typing.Union[str, int]
    '''

    pass


def layer_remove():
    ''' Remove active Grease Pencil layer

    '''

    pass


def lock_all():
    ''' Lock all Grease Pencil layers to prevent them from being accidentally modified

    '''

    pass


def lock_layer():
    ''' Lock and hide any color not used in any layer

    '''

    pass


def material_hide(unselected: bool = False):
    ''' Hide selected/unselected Grease Pencil materials

    :param unselected: Unselected, Hide unselected rather than selected colors
    :type unselected: bool
    '''

    pass


def material_isolate(affect_visibility: bool = False):
    ''' Toggle whether the active material is the only one that is editable and/or visible

    :param affect_visibility: Affect Visibility, In addition to toggling the editability, also affect the visibility
    :type affect_visibility: bool
    '''

    pass


def material_lock_all():
    ''' Lock all Grease Pencil materials to prevent them from being accidentally modified

    '''

    pass


def material_lock_unused():
    ''' Lock any material not used in any selected stroke

    '''

    pass


def material_reveal():
    ''' Unhide all hidden Grease Pencil materials

    '''

    pass


def material_select(deselect: bool = False):
    ''' Select/Deselect all Grease Pencil strokes using current material

    :param deselect: Deselect, Unselect strokes
    :type deselect: bool
    '''

    pass


def material_set(slot: typing.Union[str, int] = 'DEFAULT'):
    ''' Set active material

    :param slot: Material Slot
    :type slot: typing.Union[str, int]
    '''

    pass


def material_to_vertex_color(remove: bool = True,
                             palette: bool = True,
                             selected: bool = False,
                             threshold: int = 3):
    ''' Replace materials in strokes with Vertex Color

    :param remove: Remove Unused Materials, Remove any unused material after the conversion
    :type remove: bool
    :param palette: Create Palette, Create a new palette with colors
    :type palette: bool
    :param selected: Only Selected, Convert only selected strokes
    :type selected: bool
    :param threshold: Threshold
    :type threshold: int
    '''

    pass


def material_unlock_all():
    ''' Unlock all Grease Pencil materials so that they can be edited

    '''

    pass


def materials_copy_to_object(only_active: bool = True):
    ''' Append Materials of the active Grease Pencil to other object

    :param only_active: Only Active, Append only active material, uncheck to append all materials
    :type only_active: bool
    '''

    pass


def move_to_layer(layer: int = 0):
    ''' Move selected strokes to another layer

    :param layer: Grease Pencil Layer
    :type layer: int
    '''

    pass


def paintmode_toggle(back: bool = False):
    ''' Enter/Exit paint mode for Grease Pencil strokes

    :param back: Return to Previous Mode, Return to previous mode
    :type back: bool
    '''

    pass


def paste(type: typing.Union[str, int] = 'ACTIVE', paste_back: bool = False):
    ''' Paste previously copied strokes to active layer or to original layer

    :param type: Type
    :type type: typing.Union[str, int]
    :param paste_back: Paste on Back, Add pasted strokes behind all strokes
    :type paste_back: bool
    '''

    pass


def primitive_box(subdivision: int = 3,
                  edges: int = 1,
                  type: typing.Union[str, int] = 'BOX',
                  wait_for_input: bool = True):
    ''' Create predefined grease pencil stroke box shapes

    :param subdivision: Subdivisions, Number of subdivision by edges
    :type subdivision: int
    :param edges: Edges, Number of points by edge
    :type edges: int
    :param type: Type, Type of shape
    :type type: typing.Union[str, int]
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    '''

    pass


def primitive_circle(subdivision: int = 94,
                     edges: int = 1,
                     type: typing.Union[str, int] = 'CIRCLE',
                     wait_for_input: bool = True):
    ''' Create predefined grease pencil stroke circle shapes

    :param subdivision: Subdivisions, Number of subdivision by edges
    :type subdivision: int
    :param edges: Edges, Number of points by edge
    :type edges: int
    :param type: Type, Type of shape
    :type type: typing.Union[str, int]
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    '''

    pass


def primitive_curve(subdivision: int = 62,
                    edges: int = 1,
                    type: typing.Union[str, int] = 'CURVE',
                    wait_for_input: bool = True):
    ''' Create predefined grease pencil stroke curve shapes

    :param subdivision: Subdivisions, Number of subdivision by edges
    :type subdivision: int
    :param edges: Edges, Number of points by edge
    :type edges: int
    :param type: Type, Type of shape
    :type type: typing.Union[str, int]
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    '''

    pass


def primitive_line(subdivision: int = 6,
                   edges: int = 1,
                   type: typing.Union[str, int] = 'LINE',
                   wait_for_input: bool = True):
    ''' Create predefined grease pencil stroke lines

    :param subdivision: Subdivisions, Number of subdivision by edges
    :type subdivision: int
    :param edges: Edges, Number of points by edge
    :type edges: int
    :param type: Type, Type of shape
    :type type: typing.Union[str, int]
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    '''

    pass


def primitive_polyline(subdivision: int = 6,
                       edges: int = 1,
                       type: typing.Union[str, int] = 'POLYLINE',
                       wait_for_input: bool = True):
    ''' Create predefined grease pencil stroke polylines

    :param subdivision: Subdivisions, Number of subdivision by edges
    :type subdivision: int
    :param edges: Edges, Number of points by edge
    :type edges: int
    :param type: Type, Type of shape
    :type type: typing.Union[str, int]
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    '''

    pass


def recalc_geometry():
    ''' Update all internal geometry data

    '''

    pass


def reproject(type: typing.Union[str, int] = 'VIEW',
              keep_original: bool = False):
    ''' Reproject the selected strokes from the current viewpoint as if they had been newly drawn (e.g. to fix problems from accidental 3D cursor movement or accidental viewport changes, or for matching deforming geometry)

    :param type: Projection Type * FRONT Front -- Reproject the strokes using the X-Z plane. * SIDE Side -- Reproject the strokes using the Y-Z plane. * TOP Top -- Reproject the strokes using the X-Y plane. * VIEW View -- Reproject the strokes to end up on the same plane, as if drawn from the current viewpoint using 'Cursor' Stroke Placement. * SURFACE Surface -- Reproject the strokes on to the scene geometry, as if drawn using 'Surface' placement. * CURSOR Cursor -- Reproject the strokes using the orientation of 3D cursor.
    :type type: typing.Union[str, int]
    :param keep_original: Keep Original, Keep original strokes and create a copy before reprojecting instead of reproject them
    :type keep_original: bool
    '''

    pass


def reset_transform_fill(mode: typing.Union[str, int] = 'ALL'):
    ''' Reset any UV transformation and back to default values

    :param mode: Mode
    :type mode: typing.Union[str, int]
    '''

    pass


def reveal(select: bool = True):
    ''' Show all Grease Pencil layers

    :param select: Select
    :type select: bool
    '''

    pass


def sculpt_paint(stroke: typing.Union[
        typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.
        List['bpy.types.OperatorStrokeElement'], 'bpy_prop_collection'] = None,
                 wait_for_input: bool = True):
    ''' Apply tweaks to strokes by painting over the strokes

    :param stroke: Stroke
    :type stroke: typing.Union[typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.List['bpy.types.OperatorStrokeElement'], 'bpy_prop_collection']
    :param wait_for_input: Wait for Input, Enter a mini 'sculpt-mode' if enabled, otherwise, exit after drawing a single stroke
    :type wait_for_input: bool
    '''

    pass


def sculptmode_toggle(back: bool = False):
    ''' Enter/Exit sculpt mode for Grease Pencil strokes

    :param back: Return to Previous Mode, Return to previous mode
    :type back: bool
    '''

    pass


def segment_add(modifier: str = ""):
    ''' Add a segment to the dash modifier

    :param modifier: Modifier, Name of the modifier to edit
    :type modifier: str
    '''

    pass


def segment_move(modifier: str = "", type: typing.Union[str, int] = 'UP'):
    ''' Move the active dash segment up or down

    :param modifier: Modifier, Name of the modifier to edit
    :type modifier: str
    :param type: Type
    :type type: typing.Union[str, int]
    '''

    pass


def segment_remove(modifier: str = "", index: int = 0):
    ''' Remove the active segment from the dash modifier

    :param modifier: Modifier, Name of the modifier to edit
    :type modifier: str
    :param index: Index, Index of the segment to remove
    :type index: int
    '''

    pass


def select(extend: bool = False,
           deselect: bool = False,
           toggle: bool = False,
           deselect_all: bool = False,
           select_passthrough: bool = False,
           entire_strokes: bool = False,
           location: typing.List[int] = (0, 0),
           use_shift_extend: bool = False):
    ''' Select Grease Pencil strokes and/or stroke points

    :param extend: Extend, Extend selection instead of deselecting everything first
    :type extend: bool
    :param deselect: Deselect, Remove from selection
    :type deselect: bool
    :param toggle: Toggle Selection, Toggle the selection
    :type toggle: bool
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor
    :type deselect_all: bool
    :param select_passthrough: Only Select Unselected, Ignore the select action when the element is already selected
    :type select_passthrough: bool
    :param entire_strokes: Entire Strokes, Select entire strokes instead of just the nearest stroke vertex
    :type entire_strokes: bool
    :param location: Location, Mouse location
    :type location: typing.List[int]
    :param use_shift_extend: Extend
    :type use_shift_extend: bool
    '''

    pass


def select_all(action: typing.Union[str, int] = 'TOGGLE'):
    ''' Change selection of all Grease Pencil strokes currently visible

    :param action: Action, Selection action to execute * TOGGLE Toggle -- Toggle selection for all elements. * SELECT Select -- Select all elements. * DESELECT Deselect -- Deselect all elements. * INVERT Invert -- Invert selection of all elements.
    :type action: typing.Union[str, int]
    '''

    pass


def select_alternate(unselect_ends: bool = False):
    ''' Select alternative points in same strokes as already selected points

    :param unselect_ends: Unselect Ends, Do not select the first and last point of the stroke
    :type unselect_ends: bool
    '''

    pass


def select_box(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[str, int] = 'SET'):
    ''' Select Grease Pencil strokes within a rectangular region

    :param xmin: X Min
    :type xmin: int
    :param xmax: X Max
    :type xmax: int
    :param ymin: Y Min
    :type ymin: int
    :param ymax: Y Max
    :type ymax: int
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    :param mode: Mode * SET Set -- Set a new selection. * ADD Extend -- Extend existing selection. * SUB Subtract -- Subtract existing selection. * XOR Difference -- Invert existing selection. * AND Intersect -- Intersect existing selection.
    :type mode: typing.Union[str, int]
    '''

    pass


def select_circle(x: int = 0,
                  y: int = 0,
                  radius: int = 25,
                  wait_for_input: bool = True,
                  mode: typing.Union[str, int] = 'SET'):
    ''' Select Grease Pencil strokes using brush selection

    :param x: X
    :type x: int
    :param y: Y
    :type y: int
    :param radius: Radius
    :type radius: int
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    :param mode: Mode * SET Set -- Set a new selection. * ADD Extend -- Extend existing selection. * SUB Subtract -- Subtract existing selection.
    :type mode: typing.Union[str, int]
    '''

    pass


def select_first(only_selected_strokes: bool = False, extend: bool = False):
    ''' Select first point in Grease Pencil strokes

    :param only_selected_strokes: Selected Strokes Only, Only select the first point of strokes that already have points selected
    :type only_selected_strokes: bool
    :param extend: Extend, Extend selection instead of deselecting all other selected points
    :type extend: bool
    '''

    pass


def select_grouped(type: typing.Union[str, int] = 'LAYER'):
    ''' Select all strokes with similar characteristics

    :param type: Type * LAYER Layer -- Shared layers. * MATERIAL Material -- Shared materials.
    :type type: typing.Union[str, int]
    '''

    pass


def select_lasso(mode: typing.Union[str, int] = 'SET',
                 path: typing.
                 Union[typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.
                       List['bpy.types.OperatorMousePath'],
                       'bpy_prop_collection'] = None):
    ''' Select Grease Pencil strokes using lasso selection

    :param mode: Mode * SET Set -- Set a new selection. * ADD Extend -- Extend existing selection. * SUB Subtract -- Subtract existing selection. * XOR Difference -- Invert existing selection. * AND Intersect -- Intersect existing selection.
    :type mode: typing.Union[str, int]
    :param path: Path
    :type path: typing.Union[typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.List['bpy.types.OperatorMousePath'], 'bpy_prop_collection']
    '''

    pass


def select_last(only_selected_strokes: bool = False, extend: bool = False):
    ''' Select last point in Grease Pencil strokes

    :param only_selected_strokes: Selected Strokes Only, Only select the last point of strokes that already have points selected
    :type only_selected_strokes: bool
    :param extend: Extend, Extend selection instead of deselecting all other selected points
    :type extend: bool
    '''

    pass


def select_less():
    ''' Shrink sets of selected Grease Pencil points

    '''

    pass


def select_linked():
    ''' Select all points in same strokes as already selected points

    '''

    pass


def select_more():
    ''' Grow sets of selected Grease Pencil points

    '''

    pass


def select_random(ratio: float = 0.5,
                  seed: int = 0,
                  action: typing.Union[str, int] = 'SELECT',
                  unselect_ends: bool = False):
    ''' Select random points for non selected strokes

    :param ratio: Ratio, Portion of items to select randomly
    :type ratio: float
    :param seed: Random Seed, Seed for the random number generator
    :type seed: int
    :param action: Action, Selection action to execute * SELECT Select -- Select all elements. * DESELECT Deselect -- Deselect all elements.
    :type action: typing.Union[str, int]
    :param unselect_ends: Unselect Ends, Do not select the first and last point of the stroke
    :type unselect_ends: bool
    '''

    pass


def select_vertex_color(threshold: int = 0):
    ''' Select all points with similar vertex color of current selected

    :param threshold: Threshold, Tolerance of the selection. Higher values select a wider range of similar colors
    :type threshold: int
    '''

    pass


def selection_opacity_toggle():
    ''' Hide/Unhide selected points for Grease Pencil strokes setting alpha factor

    '''

    pass


def selectmode_toggle(mode: int = 0):
    ''' Set selection mode for Grease Pencil strokes

    :param mode: Select Mode, Select mode
    :type mode: int
    '''

    pass


def set_active_material():
    ''' Set the selected stroke material as the active material

    '''

    pass


def snap_cursor_to_selected():
    ''' Snap cursor to center of selected points

    '''

    pass


def snap_to_cursor(use_offset: bool = True):
    ''' Snap selected points/strokes to the cursor

    :param use_offset: With Offset, Offset the entire stroke instead of selected points only
    :type use_offset: bool
    '''

    pass


def snap_to_grid():
    ''' Snap selected points to the nearest grid points

    '''

    pass


def stroke_apply_thickness():
    ''' Apply the thickness change of the layer to its strokes

    '''

    pass


def stroke_arrange(direction: typing.Union[str, int] = 'UP'):
    ''' Arrange selected strokes up/down in the display order of the active layer

    :param direction: Direction
    :type direction: typing.Union[str, int]
    '''

    pass


def stroke_caps_set(type: typing.Union[str, int] = 'TOGGLE'):
    ''' Change stroke caps mode (rounded or flat)

    :param type: Type * TOGGLE Both. * START Start. * END End. * DEFAULT Default -- Set as default rounded.
    :type type: typing.Union[str, int]
    '''

    pass


def stroke_change_color(material: str = ""):
    ''' Move selected strokes to active material

    :param material: Material, Name of the material
    :type material: str
    '''

    pass


def stroke_cutter(path: typing.Union[
        typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.
        List['bpy.types.OperatorMousePath'], 'bpy_prop_collection'] = None,
                  flat_caps: bool = False):
    ''' Select section and cut

    :param path: Path
    :type path: typing.Union[typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.List['bpy.types.OperatorMousePath'], 'bpy_prop_collection']
    :param flat_caps: Flat Caps
    :type flat_caps: bool
    '''

    pass


def stroke_cyclical_set(type: typing.Union[str, int] = 'TOGGLE',
                        geometry: bool = False):
    ''' Close or open the selected stroke adding an edge from last to first point

    :param type: Type
    :type type: typing.Union[str, int]
    :param geometry: Create Geometry, Create new geometry for closing stroke
    :type geometry: bool
    '''

    pass


def stroke_editcurve_set_handle_type(
        type: typing.Union[str, int] = 'AUTOMATIC'):
    ''' Set the type of a edit curve handle

    :param type: Type, Spline type
    :type type: typing.Union[str, int]
    '''

    pass


def stroke_enter_editcurve_mode(error_threshold: float = 0.1):
    ''' Called to transform a stroke into a curve

    :param error_threshold: Error Threshold, Threshold on the maximum deviation from the actual stroke
    :type error_threshold: float
    '''

    pass


def stroke_flip():
    ''' Change direction of the points of the selected strokes

    '''

    pass


def stroke_join(type: typing.Union[str, int] = 'JOIN',
                leave_gaps: bool = False):
    ''' Join selected strokes (optionally as new stroke)

    :param type: Type
    :type type: typing.Union[str, int]
    :param leave_gaps: Leave Gaps, Leave gaps between joined strokes instead of linking them
    :type leave_gaps: bool
    '''

    pass


def stroke_merge(mode: typing.Union[str, int] = 'STROKE',
                 back: bool = False,
                 additive: bool = False,
                 cyclic: bool = False,
                 clear_point: bool = False,
                 clear_stroke: bool = False):
    ''' Create a new stroke with the selected stroke points

    :param mode: Mode
    :type mode: typing.Union[str, int]
    :param back: Draw on Back, Draw new stroke below all previous strokes
    :type back: bool
    :param additive: Additive Drawing, Add to previous drawing
    :type additive: bool
    :param cyclic: Cyclic, Close new stroke
    :type cyclic: bool
    :param clear_point: Dissolve Points, Dissolve old selected points
    :type clear_point: bool
    :param clear_stroke: Delete Strokes, Delete old selected strokes
    :type clear_stroke: bool
    '''

    pass


def stroke_merge_by_distance(threshold: float = 0.001,
                             use_unselected: bool = False):
    ''' Merge points by distance

    :param threshold: Threshold
    :type threshold: float
    :param use_unselected: Unselected, Use whole stroke, not only selected points
    :type use_unselected: bool
    '''

    pass


def stroke_merge_material(hue_threshold: float = 0.001,
                          sat_threshold: float = 0.001,
                          val_threshold: float = 0.001):
    ''' Replace materials in strokes merging similar

    :param hue_threshold: Hue Threshold
    :type hue_threshold: float
    :param sat_threshold: Saturation Threshold
    :type sat_threshold: float
    :param val_threshold: Value Threshold
    :type val_threshold: float
    '''

    pass


def stroke_normalize(mode: typing.Union[str, int] = 'THICKNESS',
                     factor: float = 1.0,
                     value: int = 10):
    ''' Normalize stroke attributes

    :param mode: Mode, Attribute to be normalized * THICKNESS Thickness -- Normalizes the stroke thickness by making all points use the same thickness value. * OPACITY Opacity -- Normalizes the stroke opacity by making all points use the same opacity value.
    :type mode: typing.Union[str, int]
    :param factor: Factor
    :type factor: float
    :param value: Value, Value
    :type value: int
    '''

    pass


def stroke_reset_vertex_color(mode: typing.Union[str, int] = 'BOTH'):
    ''' Reset vertex color for all or selected strokes

    :param mode: Mode * STROKE Stroke -- Reset Vertex Color to Stroke only. * FILL Fill -- Reset Vertex Color to Fill only. * BOTH Stroke & Fill -- Reset Vertex Color to Stroke and Fill.
    :type mode: typing.Union[str, int]
    '''

    pass


def stroke_sample(length: float = 0.1):
    ''' Sample stroke points to predefined segment length

    :param length: Length
    :type length: float
    '''

    pass


def stroke_separate(mode: typing.Union[str, int] = 'POINT'):
    ''' Separate the selected strokes or layer in a new grease pencil object

    :param mode: Mode * POINT Selected Points -- Separate the selected points. * STROKE Selected Strokes -- Separate the selected strokes. * LAYER Active Layer -- Separate the strokes of the current layer.
    :type mode: typing.Union[str, int]
    '''

    pass


def stroke_simplify(factor: float = 0.0):
    ''' Simplify selected stroked reducing number of points

    :param factor: Factor
    :type factor: float
    '''

    pass


def stroke_simplify_fixed(step: int = 1):
    ''' Simplify selected stroked reducing number of points using fixed algorithm

    :param step: Steps, Number of simplify steps
    :type step: int
    '''

    pass


def stroke_smooth(repeat: int = 2,
                  factor: float = 1.0,
                  only_selected: bool = True,
                  smooth_position: bool = True,
                  smooth_thickness: bool = True,
                  smooth_strength: bool = False,
                  smooth_uv: bool = False):
    ''' Smooth selected strokes

    :param repeat: Repeat
    :type repeat: int
    :param factor: Factor
    :type factor: float
    :param only_selected: Selected Points, Smooth only selected points in the stroke
    :type only_selected: bool
    :param smooth_position: Position
    :type smooth_position: bool
    :param smooth_thickness: Thickness
    :type smooth_thickness: bool
    :param smooth_strength: Strength
    :type smooth_strength: bool
    :param smooth_uv: UV
    :type smooth_uv: bool
    '''

    pass


def stroke_split():
    ''' Split selected points as new stroke on same frame

    '''

    pass


def stroke_subdivide(number_cuts: int = 1,
                     factor: float = 0.0,
                     repeat: int = 1,
                     only_selected: bool = True,
                     smooth_position: bool = True,
                     smooth_thickness: bool = True,
                     smooth_strength: bool = False,
                     smooth_uv: bool = False):
    ''' Subdivide between continuous selected points of the stroke adding a point half way between them

    :param number_cuts: Number of Cuts
    :type number_cuts: int
    :param factor: Smooth
    :type factor: float
    :param repeat: Repeat
    :type repeat: int
    :param only_selected: Selected Points, Smooth only selected points in the stroke
    :type only_selected: bool
    :param smooth_position: Position
    :type smooth_position: bool
    :param smooth_thickness: Thickness
    :type smooth_thickness: bool
    :param smooth_strength: Strength
    :type smooth_strength: bool
    :param smooth_uv: UV
    :type smooth_uv: bool
    '''

    pass


def stroke_trim():
    ''' Trim selected stroke to first loop or intersection

    '''

    pass


def tint_flip():
    ''' Switch tint colors :file: startup/bl_ui/properties_grease_pencil_common.py\:877 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_ui/properties_grease_pencil_common.py$877> _

    '''

    pass


def trace_image(target: typing.Union[str, int] = 'NEW',
                thickness: int = 10,
                resolution: int = 5,
                scale: float = 1.0,
                sample: float = 0.0,
                threshold: float = 0.5,
                turnpolicy: typing.Union[str, int] = 'MINORITY',
                mode: typing.Union[str, int] = 'SINGLE',
                use_current_frame: bool = True):
    ''' Extract Grease Pencil strokes from image

    :param target: Target Object, Target grease pencil
    :type target: typing.Union[str, int]
    :param thickness: Thickness
    :type thickness: int
    :param resolution: Resolution, Resolution of the generated curves
    :type resolution: int
    :param scale: Scale, Scale of the final stroke
    :type scale: float
    :param sample: Sample, Distance to sample points, zero to disable
    :type sample: float
    :param threshold: Color Threshold, Determine the lightness threshold above which strokes are generated
    :type threshold: float
    :param turnpolicy: Turn Policy, Determines how to resolve ambiguities during decomposition of bitmaps into paths * BLACK Black -- Prefers to connect black (foreground) components. * WHITE White -- Prefers to connect white (background) components. * LEFT Left -- Always take a left turn. * RIGHT Right -- Always take a right turn. * MINORITY Minority -- Prefers to connect the color (black or white) that occurs least frequently in the local neighborhood of the current position. * MAJORITY Majority -- Prefers to connect the color (black or white) that occurs most frequently in the local neighborhood of the current position. * RANDOM Random -- Choose pseudo-randomly.
    :type turnpolicy: typing.Union[str, int]
    :param mode: Mode, Determines if trace simple image or full sequence * SINGLE Single -- Trace the current frame of the image. * SEQUENCE Sequence -- Trace full sequence.
    :type mode: typing.Union[str, int]
    :param use_current_frame: Start At Current Frame, Trace Image starting in current image frame
    :type use_current_frame: bool
    '''

    pass


def transform_fill(mode: typing.Union[str, int] = 'ROTATE',
                   location: typing.List[float] = (0.0, 0.0),
                   rotation: float = 0.0,
                   scale: float = 0.0,
                   release_confirm: bool = False):
    ''' Transform grease pencil stroke fill

    :param mode: Mode
    :type mode: typing.Union[str, int]
    :param location: Location
    :type location: typing.List[float]
    :param rotation: Rotation
    :type rotation: float
    :param scale: Scale
    :type scale: float
    :param release_confirm: Confirm on Release
    :type release_confirm: bool
    '''

    pass


def unlock_all():
    ''' Unlock all Grease Pencil layers so that they can be edited

    '''

    pass


def vertex_color_brightness_contrast(mode: typing.Union[str, int] = 'BOTH',
                                     brightness: float = 0.0,
                                     contrast: float = 0.0):
    ''' Adjust vertex color brightness/contrast

    :param mode: Mode
    :type mode: typing.Union[str, int]
    :param brightness: Brightness
    :type brightness: float
    :param contrast: Contrast
    :type contrast: float
    '''

    pass


def vertex_color_hsv(mode: typing.Union[str, int] = 'BOTH',
                     h: float = 0.5,
                     s: float = 1.0,
                     v: float = 1.0):
    ''' Adjust vertex color HSV values

    :param mode: Mode
    :type mode: typing.Union[str, int]
    :param h: Hue
    :type h: float
    :param s: Saturation
    :type s: float
    :param v: Value
    :type v: float
    '''

    pass


def vertex_color_invert(mode: typing.Union[str, int] = 'BOTH'):
    ''' Invert RGB values

    :param mode: Mode
    :type mode: typing.Union[str, int]
    '''

    pass


def vertex_color_levels(mode: typing.Union[str, int] = 'BOTH',
                        offset: float = 0.0,
                        gain: float = 1.0):
    ''' Adjust levels of vertex colors

    :param mode: Mode
    :type mode: typing.Union[str, int]
    :param offset: Offset, Value to add to colors
    :type offset: float
    :param gain: Gain, Value to multiply colors by
    :type gain: float
    '''

    pass


def vertex_color_set(mode: typing.Union[str, int] = 'BOTH',
                     factor: float = 1.0):
    ''' Set active color to all selected vertex

    :param mode: Mode
    :type mode: typing.Union[str, int]
    :param factor: Factor, Mix Factor
    :type factor: float
    '''

    pass


def vertex_group_assign():
    ''' Assign the selected vertices to the active vertex group

    '''

    pass


def vertex_group_deselect():
    ''' Deselect all selected vertices assigned to the active vertex group

    '''

    pass


def vertex_group_invert():
    ''' Invert weights to the active vertex group

    '''

    pass


def vertex_group_normalize():
    ''' Normalize weights to the active vertex group

    '''

    pass


def vertex_group_normalize_all(lock_active: bool = True):
    ''' Normalize all weights of all vertex groups, so that for each vertex, the sum of all weights is 1.0

    :param lock_active: Lock Active, Keep the values of the active group while normalizing others
    :type lock_active: bool
    '''

    pass


def vertex_group_remove_from():
    ''' Remove the selected vertices from active or all vertex group(s)

    '''

    pass


def vertex_group_select():
    ''' Select all the vertices assigned to the active vertex group

    '''

    pass


def vertex_group_smooth(factor: float = 0.5, repeat: int = 1):
    ''' Smooth weights to the active vertex group

    :param factor: Factor
    :type factor: float
    :param repeat: Iterations
    :type repeat: int
    '''

    pass


def vertex_paint(stroke: typing.Union[
        typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.
        List['bpy.types.OperatorStrokeElement'], 'bpy_prop_collection'] = None,
                 wait_for_input: bool = True):
    ''' Paint stroke points with a color

    :param stroke: Stroke
    :type stroke: typing.Union[typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.List['bpy.types.OperatorStrokeElement'], 'bpy_prop_collection']
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    '''

    pass


def vertexmode_toggle(back: bool = False):
    ''' Enter/Exit vertex paint mode for Grease Pencil strokes

    :param back: Return to Previous Mode, Return to previous mode
    :type back: bool
    '''

    pass


def weight_paint(stroke: typing.Union[
        typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.
        List['bpy.types.OperatorStrokeElement'], 'bpy_prop_collection'] = None,
                 wait_for_input: bool = True):
    ''' Paint stroke points with a color

    :param stroke: Stroke
    :type stroke: typing.Union[typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.List['bpy.types.OperatorStrokeElement'], 'bpy_prop_collection']
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    '''

    pass


def weightmode_toggle(back: bool = False):
    ''' Enter/Exit weight paint mode for Grease Pencil strokes

    :param back: Return to Previous Mode, Return to previous mode
    :type back: bool
    '''

    pass
